@@ -9,98 +9,33 @@ def main():
     import numpy as np
     import cv2
 
-    from PySide6.QtWidgets import QApplication, QMainWindow, QFileDialog, QMessageBox, \
-        QGraphicsView, QGraphicsScene, QGraphicsPixmapItem, QFrame
-    from PySide6.QtCore import Qt, QObject, Slot, Signal, QPoint, QPointF, QRectF
-    from PySide6.QtGui import QPixmap, QImage, QPainter, QBrush, QColor, QPolygonF
+    from PySide6.QtWidgets import QApplication, QMainWindow, QFileDialog, QMessageBox
+    from PySide6.QtCore import Qt, QObject, Slot, Signal, QSettings, QPointF
+    from PySide6.QtGui import QPixmap, QImage, QPainter, QColor, QPolygonF
     from src.ui_mainwindow import Ui_MainWindow
+    from src.viewer import ImageViewer
     from src.utils import sort_cw, crop_module
 
 
-    class ImageViewer(QGraphicsView):
-        imageClicked = Signal(QPoint)
-
-        def __init__(self, parent):
-            super(ImageViewer, self).__init__(parent)
-            self._zoom = 0
-            self._empty = True
-            self._scene = QGraphicsScene(self)
-            self._image = QGraphicsPixmapItem()
-            self._scene.addItem(self._image)
-            self.setScene(self._scene)
-            self.setDragMode(QGraphicsView.NoDrag)
-            self.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)
-            self.setResizeAnchor(QGraphicsView.AnchorUnderMouse)
-            self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-            self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-            self.setBackgroundBrush(QBrush(QColor(30, 30, 30)))
-            self.setFrameShape(QFrame.NoFrame)
-
-        def hasImage(self):
-            return not self._empty
-
-        def fitInView(self, scale=True):
-            rect = QRectF(self._image.pixmap().rect())
-            if not rect.isNull():
-                self.setSceneRect(rect)
-                if self.hasImage():
-                    unity = self.transform().mapRect(QRectF(0, 0, 1, 1))
-                    self.scale(1 / unity.width(), 1 / unity.height())
-                    viewrect = self.viewport().rect()
-                    scenerect = self.transform().mapRect(rect)
-                    factor = min(viewrect.width() / scenerect.width(),
-                                viewrect.height() / scenerect.height())
-                    self.scale(factor, factor)
-                self._zoom = 0
-
-        def setImage(self, pixmap=None, fit_in_view=True):
-            if fit_in_view:
-                self._zoom = 0
-            if pixmap and not pixmap.isNull():
-                self._empty = False
-                self._image.setPixmap(pixmap)
-            else:
-                self._empty = True
-                self._image.setPixmap(QPixmap())
-            if fit_in_view:
-                self.fitInView()
-
-        def wheelEvent(self, event):
-            if self.hasImage():
-                if event.angleDelta().y() > 0:
-                    factor = 1.25
-                    self._zoom += 1
-                else:
-                    factor = 0.8
-                    self._zoom -= 1
-                if self._zoom > 0:
-                    self.scale(factor, factor)
-                elif self._zoom == 0:
-                    self.fitInView()
-                else:
-                    self._zoom = 0
-
-        def mousePressEvent(self, event):
-            if self._image.isUnderMouse():
-                self.imageClicked.emit(self.mapToScene(event.pos()).toPoint())
-            super(ImageViewer, self).mousePressEvent(event)
-
-
 
     class MainWindow(QMainWindow):
         def __init__(self):
             super(MainWindow, self).__init__()
             self.ui = Ui_MainWindow()
             self.ui.setupUi(self)
-            self.setWindowTitle("QuadCropper")
-            self.disable()
+            self.setWindowTitle("QuadCropper")            
             self.model = Model()
+            self.disable()
 
             # image viewer
             self.viewer = ImageViewer(self)
             self.viewer.imageClicked.connect(self.imageClicked)        
             self.ui.gridLayout.addWidget(self.viewer, 0, 1, 1, 1)
 
+            # application presets
+            self.settings = QSettings('Lukas Bommes', 'QuadCropper')
+
+            # signals
             self.model.current_image_changed.connect(lambda _: self.update_image_label(fit_in_view=True))
             self.model.current_quad_changed.connect(lambda _: self.update_image_label(fit_in_view=False))
             self.model.quads_changed.connect(self.save_quads)
@@ -110,39 +45,56 @@ def main():
             self.model.quads_changed.connect(self.update_quad_list)
             self.model.selected_quad_changed.connect(self.selected_quad_changed)
             self.model.selected_quad_changed.connect(lambda _: self.update_image_label(fit_in_view=False))
+            self.ui.autoPatchSizeCheckbox.stateChanged.connect(lambda value: setattr(self.model, 'auto_patch_size', bool(value)))
+            self.model.auto_patch_size_changed.connect(lambda value: self.auto_patch_size_changed(value))
+            self.model.auto_patch_size_changed.connect(self.ui.autoPatchSizeCheckbox.setChecked)
+            self.model.patch_width_changed.connect(self.ui.patchWidthSpinBox.setValue)
+            self.ui.patchWidthSpinBox.valueChanged.connect(lambda value: setattr(self.model, 'patch_width', value))
+            self.model.patch_height_changed.connect(self.ui.patchHeightSpinBox.setValue)
+            self.ui.patchHeightSpinBox.valueChanged.connect(lambda value: setattr(self.model, 'patch_height', value))
 
             # menu actions
             self.ui.actionOpen_Folder.triggered.connect(self.open_folder)
             self.ui.actionClose_Folder.triggered.connect(self.close_folder)
-            self.ui.actionClear_all_quadrilaterals.triggered.connect(self.clear_all_quadrilaterals)
-            self.ui.actionCrop_All.triggered.connect(self.crop_all)
             self.ui.actionAbout.triggered.connect(self.about)
             
             # buttons
             self.ui.deleteSelectedQuadButton.clicked.connect(self.delete_selected_quad_button_clicked)
             self.ui.deleteAllQuadsButton.clicked.connect(self.delete_all_quads_button_clicked)
+            self.ui.cropAllButton.clicked.connect(self.crop_all)
 
             # image selection changed
             self.ui.imagesListWidget.currentItemChanged.connect(self.image_selection_changed)
             self.ui.quadsListWidget.currentItemChanged.connect(self.quad_selection_changed)
 
+            # restore settings
+            self.restore_settings()
+
 
         def enable(self):
             self.ui.actionOpen_Folder.setEnabled(False)
             self.ui.actionClose_Folder.setEnabled(True)
-            self.ui.actionCrop_All.setEnabled(True)
-            self.ui.actionClear_all_quadrilaterals.setEnabled(True)
             self.ui.deleteSelectedQuadButton.setEnabled(False)
             self.ui.deleteAllQuadsButton.setEnabled(False)
+            self.ui.cropAllButton.setEnabled(True)
+            self.ui.autoPatchSizeCheckbox.setEnabled(True)
+            self.ui.patchWidthSpinBox.setEnabled(False)
+            self.ui.patchHeightSpinBox.setEnabled(False)
+            self.ui.patchHeightLabel.setEnabled(False)
+            self.ui.patchWidthLabel.setEnabled(False)
 
         
         def disable(self):
             self.ui.actionOpen_Folder.setEnabled(True)
-            self.ui.actionClose_Folder.setEnabled(False)
-            self.ui.actionCrop_All.setEnabled(False)
-            self.ui.actionClear_all_quadrilaterals.setEnabled(False)
+            self.ui.actionClose_Folder.setEnabled(False)            
             self.ui.deleteSelectedQuadButton.setEnabled(False)
             self.ui.deleteAllQuadsButton.setEnabled(False)
+            self.ui.cropAllButton.setEnabled(False)
+            self.ui.autoPatchSizeCheckbox.setEnabled(False)
+            self.ui.patchWidthSpinBox.setEnabled(False)
+            self.ui.patchHeightSpinBox.setEnabled(False)
+            self.ui.patchHeightLabel.setEnabled(False)
+            self.ui.patchWidthLabel.setEnabled(False)
 
 
         def resizeEvent(self, event):
@@ -155,6 +107,13 @@ def main():
             event.accept()
 
 
+        def closeEvent(self, event):
+            self.settings.setValue('window size', self.size())
+            self.settings.setValue('window position', self.pos())
+            print("Saved presets")
+            event.accept()
+
+
         def about(self):
             gh = "LukasBommes/Quad-Cropper"
             about_text = "QuadCropper<br><br>" \
@@ -168,7 +127,15 @@ def main():
             )
 
 
-        @Slot()
+        def restore_settings(self):
+            window_size = self.settings.value('window size')
+            if window_size:
+                self.resize(window_size)
+            window_position = self.settings.value('window position')
+            if window_position:          
+                self.move(window_position)
+
+
         def open_folder(self):
             dir = QFileDialog.getExistingDirectory(
                 self, caption="Open Dataset", options=QFileDialog.ShowDirsOnly)
@@ -186,38 +153,12 @@ def main():
             print("Dataset opened")
 
 
-        @Slot()
         def close_folder(self):
             self.model.reset()
+            print(self.model.auto_patch_size)
             self.disable()
             print("Dataset closed")
 
-        
-        @Slot()
-        def clear_all_quadrilaterals(self):
-            if not self.model.image_dir:
-                return
-            # show confirmation dialog and if success try to delete meta.json
-            delete_dialog_title = "Delete All Quadrilaterals"
-            delete_dialog_text = ("Are you sure you want to permanetely delete all {} "
-                                  "annotated quadrilaterals from all images in {}?".format(
-                                  self.get_num_quads(), self.model.image_dir))
-            delete_dialog = QMessageBox(
-                QMessageBox.Question, 
-                delete_dialog_title, 
-                delete_dialog_text, 
-                QMessageBox.Yes|QMessageBox.No)            
-            if delete_dialog.exec() == QMessageBox.Yes:
-                quads_file = os.path.join(self.model.image_dir, "meta.json")
-                try:
-                    os.remove(quads_file)
-                except FileNotFoundError:
-                    print("No meta file found in {}. Skipping deletion.".format(quads_file))
-                else:
-                    self.model.quads = defaultdict(dict)
-                    self.model.current_quad = []
-                    self.model.selected_quad = None
-
 
         def get_num_quads(self, image_file=None):
             num_quads = 0
@@ -231,7 +172,6 @@ def main():
             return num_quads
 
 
-        @Slot()
         def crop_all(self):
             # select output directory
             output_dir = QFileDialog.getExistingDirectory(
@@ -239,6 +179,12 @@ def main():
             if output_dir == "":
                 return
             # crop and rectify annotated regions from current image
+            if self.model.auto_patch_size:
+                crop_width = None
+                crop_aspect = None
+            else:
+                crop_width = self.model.patch_width
+                crop_aspect = self.model.patch_height / self.model.patch_width
             for image_name, quads in self.model.quads.items():
                 image_file = os.path.join(self.model.image_dir, image_name)
                 image = self.load_image(image_file)
@@ -246,20 +192,23 @@ def main():
                 for quad_id, quad in quads.items():
                     quad = sort_cw(np.array(quad))
                     quad = quad.reshape(4, 1, 2)
-                    image_cropped, _ = crop_module(image, quad, crop_width=None, crop_aspect=None, rotate_mode=None)
-                    cv2.imwrite(os.path.join(output_dir, "{}_{}{}".format(image_file_name, quad_id, image_file_ext)), image_cropped)
+                    image_cropped, _ = crop_module(
+                        image, quad, crop_width=crop_width, crop_aspect=crop_aspect, rotate_mode=None)
+                    cv2.imwrite(os.path.join(output_dir, "{}_{}{}".format(
+                        image_file_name, quad_id, image_file_ext)), image_cropped)
             print("Cropped annotated quads for all images in opened folder")
 
 
         def get_image_files(self):
             image_files = []
-            for file_type in ["jpg", "JPG", "png", "PNG", "tiff", "TIFF"]:
+            for file_type in ["jpg", "jpeg", "png"]:
                 files = sorted(glob.glob(os.path.join(self.model.image_dir, "*.{}".format(file_type))))
+                files_upper = sorted(glob.glob(os.path.join(self.model.image_dir, "*.{}".format(file_type.upper()))))
                 image_files.extend(files)
-            self.model.image_files = image_files
+                image_files.extend(files_upper)
+            self.model.image_files = list(set(image_files))  # remove duplicates
 
         
-        @Slot()
         def update_image_list(self, image_files):
             self.ui.imagesListWidget.clear()
             for image_file in image_files:
@@ -267,7 +216,6 @@ def main():
                 self.ui.imagesListWidget.addItem(file_name)
 
 
-        @Slot()
         def image_selection_changed(self):
             self.model.current_quad = []
             self.model.current_image = None
@@ -295,7 +243,6 @@ def main():
             return cv2.imread(image_file)
 
 
-        @Slot()
         def update_image_label(self, fit_in_view):
             if not self.model.current_image:
                 self.viewer.setImage(None)
@@ -362,7 +309,6 @@ def main():
             #print("self.model.quads: ", self.model.quads)
 
         
-        @Slot()
         def save_quads(self):
             if not self.model.image_dir:
                 return
@@ -371,7 +317,6 @@ def main():
                 json.dump(self.model.quads, file)
 
 
-        @Slot()
         def update_quad_list(self):
             self.ui.quadsListWidget.clear()
             if not self.model.current_image:
@@ -391,7 +336,6 @@ def main():
                     self.ui.deleteAllQuadsButton.setEnabled(False)
 
 
-        @Slot()
         def quad_selection_changed(self):
             # get selected quad from list
             self.model.selected_quad = None
@@ -401,7 +345,6 @@ def main():
             self.model.selected_quad = current.text()
 
 
-        @Slot()
         def selected_quad_changed(self):
             if self.model.selected_quad:
                 self.ui.deleteSelectedQuadButton.setEnabled(True)
@@ -409,7 +352,6 @@ def main():
                 self.ui.deleteSelectedQuadButton.setEnabled(False)
 
 
-        @Slot()
         def delete_selected_quad_button_clicked(self):
             if not self.model.current_image:
                 return
@@ -427,7 +369,6 @@ def main():
                 self.model.quads = quads_copy
 
         
-        @Slot()
         def delete_all_quads_button_clicked(self):
             if not self.model.image_dir:
                 return
@@ -441,6 +382,11 @@ def main():
                 self.model.quads = quads_copy
 
 
+        def auto_patch_size_changed(self, auto_patch_size):
+            self.ui.patchWidthSpinBox.setEnabled(not auto_patch_size)
+            self.ui.patchHeightSpinBox.setEnabled(not auto_patch_size)
+
+
 
     class Model(QObject):
         quads_changed = Signal(object)
@@ -448,6 +394,9 @@ def main():
         current_image_changed = Signal(object)
         image_files_changed = Signal(list)
         selected_quad_changed = Signal(str)
+        auto_patch_size_changed = Signal(bool)
+        patch_width_changed = Signal(int)
+        patch_height_changed = Signal(int)
 
         def __init__(self):
             super().__init__()
@@ -457,6 +406,9 @@ def main():
             self._current_quad = []
             self._current_image = None
             self._selected_quad = None
+            self._auto_patch_size = True
+            self._patch_width = 0
+            self._patch_height = 0
 
         def reset(self):
             self.image_dir = None
@@ -465,6 +417,9 @@ def main():
             self.current_quad = []
             self.current_image = None
             self.selected_quad = None
+            self.auto_patch_size = True
+            self.patch_width = 0
+            self.patch_height = 0
 
         @property
         def image_dir(self):
@@ -524,11 +479,36 @@ def main():
             self.selected_quad_changed.emit(value)
             #print("selected_quad_changed emitted")
 
+        @property
+        def auto_patch_size(self):
+            return self._auto_patch_size
+
+        @auto_patch_size.setter
+        def auto_patch_size(self, value):
+            self._auto_patch_size = value
+            self.auto_patch_size_changed.emit(value)
+
+        @property
+        def patch_width(self):
+            return self._patch_width
+
+        @patch_width.setter
+        def patch_width(self, value):
+            self._patch_width = value
+            self.patch_width_changed.emit(value)
+
+        @property
+        def patch_height(self):
+            return self._patch_height
+
+        @patch_height.setter
+        def patch_height(self, value):
+            self._patch_height = value
+            self.patch_height_changed.emit(value)
+
 
     app = QApplication(sys.argv)
     window = MainWindow()
-    screen = window.screen()
-    window.resize(screen.availableSize() * 0.5)
     window.show()
 
     sys.exit(app.exec())
